@@ -15,7 +15,7 @@ import re
 import sys
 
 from unittest import mock
-from typing import Any, Dict, IO, List, Optional, Set, Tuple
+from typing import Any, IO
 
 import pytest
 
@@ -56,8 +56,8 @@ class HtmlTestHelpers(CoverageTest):
 
     def run_coverage(
         self,
-        covargs: Optional[Dict[str, Any]] = None,
-        htmlargs: Optional[Dict[str, Any]] = None,
+        covargs: dict[str, Any] | None = None,
+        htmlargs: dict[str, Any] | None = None,
     ) -> float:
         """Run coverage.py on main_file.py, and create an HTML report."""
         self.clean_local_file_imports()
@@ -109,7 +109,7 @@ class HtmlTestHelpers(CoverageTest):
         )
 
     def assert_valid_hrefs(self) -> None:
-        """Assert that the hrefs in htmlcov/*.html to see the references are valid.
+        """Assert that the hrefs in htmlcov/*.html are valid.
 
         Doesn't check external links (those with a protocol).
         """
@@ -125,6 +125,7 @@ class HtmlTestHelpers(CoverageTest):
                     continue
                 if "://" in href:
                     continue
+                href = href.partition("#")[0]   # ignore fragment in URLs.
                 hrefs[href].add(fname)
         for href, sources in hrefs.items():
             assert os.path.exists(f"htmlcov/{href}"), (
@@ -134,7 +135,7 @@ class HtmlTestHelpers(CoverageTest):
 
 class FileWriteTracker:
     """A fake object to track how `open` is used to write files."""
-    def __init__(self, written: Set[str]) -> None:
+    def __init__(self, written: set[str]) -> None:
         self.written = written
 
     def open(self, filename: str, mode: str = "r") -> IO[str]:
@@ -155,12 +156,12 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         self.real_coverage_version = coverage.__version__
         self.addCleanup(setattr, coverage, "__version__", self.real_coverage_version)
 
-        self.files_written: Set[str]
+        self.files_written: set[str]
 
     def run_coverage(
         self,
-        covargs: Optional[Dict[str, Any]] = None,
-        htmlargs: Optional[Dict[str, Any]] = None,
+        covargs: dict[str, Any] | None = None,
+        htmlargs: dict[str, Any] | None = None,
     ) -> float:
         """Run coverage in-process for the delta tests.
 
@@ -179,6 +180,8 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
     def assert_htmlcov_files_exist(self) -> None:
         """Assert that all the expected htmlcov files exist."""
         self.assert_exists("htmlcov/index.html")
+        self.assert_exists("htmlcov/function_index.html")
+        self.assert_exists("htmlcov/class_index.html")
         self.assert_exists("htmlcov/main_file_py.html")
         self.assert_exists("htmlcov/helper1_py.html")
         self.assert_exists("htmlcov/helper2_py.html")
@@ -310,7 +313,7 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         with open("htmlcov/status.json") as status_json:
             status_data = json.load(status_json)
 
-        assert status_data['format'] == 2
+        assert status_data['format'] == 5
         status_data['format'] = 99
         with open("htmlcov/status.json", "w") as status_json:
             json.dump(status_data, status_json)
@@ -616,6 +619,26 @@ class HtmlTest(HtmlTestHelpers, CoverageTest):
         res = self.run_coverage(covargs=dict(source="."), htmlargs=dict(skip_covered=True))
         assert res == 100.0
         self.assert_doesnt_exist("htmlcov/main_file_py.html")
+        # Since there are no files to report, we can't collect any region
+        # information, so there are no region-based index pages.
+        self.assert_doesnt_exist("htmlcov/function_index.html")
+        self.assert_doesnt_exist("htmlcov/class_index.html")
+
+    def test_report_skip_covered_100_functions(self) -> None:
+        self.make_file("main_file.py", """\
+            def normal():
+                print("z")
+            def abnormal():
+                print("a")
+            normal()
+        """)
+        res = self.run_coverage(covargs=dict(source="."), htmlargs=dict(skip_covered=True))
+        assert res == 80.0
+        self.assert_exists("htmlcov/main_file_py.html")
+        # We have a file to report, so we get function and class index pages,
+        # even though there are no classes.
+        self.assert_exists("htmlcov/function_index.html")
+        self.assert_exists("htmlcov/class_index.html")
 
     def make_init_and_main(self) -> None:
         """Helper to create files for skip_empty scenarios."""
@@ -659,7 +682,7 @@ def filepath_to_regex(path: str) -> str:
 def compare_html(
     expected: str,
     actual: str,
-    extra_scrubs: Optional[List[Tuple[str, str]]] = None,
+    extra_scrubs: list[tuple[str, str]] | None = None,
 ) -> None:
     """Specialized compare function for our HTML files."""
     __tracebackhide__ = True    # pytest, please don't show me this function.
@@ -718,6 +741,10 @@ class HtmlGoldTest(CoverageTest):
             '<td class="right" data-ratio="2 3">67%</td>',
         )
 
+    @pytest.mark.skipif(
+        env.PYPY and env.PYVERSION[:2] == (3, 8),
+        reason="PyPy 3.8 produces different results!?",
+    )
     def test_b_branch(self) -> None:
         self.make_file("b.py", """\
             def one(x):
@@ -937,7 +964,7 @@ assert len(math) == 18
         compare_html(
             gold_path("html/other"), "out/other",
             extra_scrubs=[
-                (r'href="d_[0-9a-z]{16}_', 'href="_TEST_TMPDIR_othersrc_'),
+                (r'href="z_[0-9a-z]{16}_', 'href="_TEST_TMPDIR_othersrc_'),
             ],
         )
         contains(
@@ -1129,10 +1156,10 @@ assert len(math) == 18
         cov = coverage.Coverage()
         cov.load()
         cov.html_report()
-        self.assert_exists("htmlcov/d_5786906b6f0ffeb4_accented_py.html")
+        self.assert_exists("htmlcov/z_5786906b6f0ffeb4_accented_py.html")
         with open("htmlcov/index.html") as indexf:
             index = indexf.read()
-        expected = '<a href="d_5786906b6f0ffeb4_accented_py.html">&#226;%saccented.py</a>'
+        expected = '<a href="z_5786906b6f0ffeb4_accented_py.html">&#226;%saccented.py</a>'
         assert expected % os.sep in index
 
 
@@ -1208,7 +1235,7 @@ class HtmlWithContextsTest(HtmlTestHelpers, CoverageTest):
         d = self.html_data_from_cov(cov, mod)
 
         context_labels = [self.EMPTY, 'two_tests.test_one', 'two_tests.test_two']
-        expected_lines: List[List[TLineNo]] = [[], self.TEST_ONE_LINES, []]
+        expected_lines: list[list[TLineNo]] = [[], self.TEST_ONE_LINES, []]
         for label, expected in zip(context_labels, expected_lines):
             actual = [ld.number for ld in d.lines if label in (ld.contexts or ())]
             assert sorted(expected) == sorted(actual)
